import * as React from 'react';
import { ISlotTypes, IComposable, IProcessResult, IGenericProps, IResolvedSlot, IResolvedSlots, IPropFilter, INativeSlotType } from './Composable.types';
/**
 * Process the slots on a composable, passing in props targeted at each entry and then creating the resolved slot collection
 * that is ready to use in a render function
 *
 * @param composable - composable component to process the slots for
 * @param info - slot info object which will be combined with resolved props on return
 * @param slotProps - props to pass into each slot
 */
export declare function useSlotProcessing(composable: IComposable, info: IProcessResult, theme: object): IResolvedSlots | undefined;
/**
 * Process a component tree and return a resolved slot for it.  This should be used as the root call for processing
 * such that the recursion handles automatically
 *
 * @param composable - component to process the hierarchy for
 * @param props - input props to pass into the component hierarchy
 */
export declare function useProcessComposableTree(composable: IComposable, props: IGenericProps, theme: object): IResolvedSlot;
/**
 * Render a slot according to the values stored in the slot info object
 *
 * @param slot - slot to perform standard rendering for
 * @param props - props for that slot, same pattern as for React.createElement
 * @param children - standard children values, as appropriate to pass to React.createElement
 */
export declare function renderSlot(slot: IResolvedSlot, ...children: React.ReactNode[]): JSX.Element | null;
/**
 * Take a non-composable component type or function and wrap it as a composable component
 *
 * @param component - type of component to wrap, either a standard react type or a function that takes props and children
 */
export declare function wrapStockComponent(component: INativeSlotType, filter?: IPropFilter): IComposable;
/**
 * turn a set of slot types into a set of IComposable interfaces
 *
 * @param slots - set of slot types to either wrap in a stock component or embed directly
 */
export declare function wrapSlots(slots: ISlotTypes): {
    [key: string]: IComposable;
};
