var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import * as React from 'react';
/**
 * Process the slots on a composable, passing in props targeted at each entry and then creating the resolved slot collection
 * that is ready to use in a render function
 *
 * @param composable - composable component to process the slots for
 * @param info - slot info object which will be combined with resolved props on return
 * @param slotProps - props to pass into each slot
 */
export function useSlotProcessing(composable, info, theme) {
    var slotProps = info.slotProps || {};
    var componentSlots = composable.slots;
    if (componentSlots) {
        var slotResults = {};
        for (var key in componentSlots) {
            if (componentSlots[key]) {
                // eslint-disable-next-line @typescript-eslint/no-use-before-define
                slotResults[key] = useProcessComposableTree(componentSlots[key], slotProps[key] || {}, theme);
            }
        }
        return slotResults;
    }
    return undefined;
}
/**
 * Process a component tree and return a resolved slot for it.  This should be used as the root call for processing
 * such that the recursion handles automatically
 *
 * @param composable - component to process the hierarchy for
 * @param props - input props to pass into the component hierarchy
 */
export function useProcessComposableTree(composable, props, theme) {
    var info = composable.useProcessProps(props, theme);
    return __assign({}, info, { slots: useSlotProcessing(composable, info, theme), composable: composable });
}
/**
 * Render a slot according to the values stored in the slot info object
 *
 * @param slot - slot to perform standard rendering for
 * @param props - props for that slot, same pattern as for React.createElement
 * @param children - standard children values, as appropriate to pass to React.createElement
 */
export function renderSlot(slot) {
    var _a;
    var children = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        children[_i - 1] = arguments[_i];
    }
    return slot.composable ? (_a = slot.composable).render.apply(_a, [slot].concat(children)) : null;
}
/**
 * Process function for standard components
 *
 * @param props - props to put into the right format for return
 * @param _theme - theme, unused for this purpose
 * @param filter - optional filter function
 */
var _stockProcessor = function (props, _theme, filter) {
    return {
        props: filter ? filter(props) : props
    };
};
/**
 * Take a non-composable component type or function and wrap it as a composable component
 *
 * @param component - type of component to wrap, either a standard react type or a function that takes props and children
 */
export function wrapStockComponent(component, filter) {
    return {
        useProcessProps: filter
            ? function (props, theme) {
                return _stockProcessor(props, theme, filter);
            }
            : _stockProcessor,
        render: function (slotInfo) {
            var children = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                children[_i - 1] = arguments[_i];
            }
            return React.createElement.apply(React, [component, slotInfo.props].concat(children));
        }
    };
}
/**
 * turn a set of slot types into a set of IComposable interfaces
 *
 * @param slots - set of slot types to either wrap in a stock component or embed directly
 */
export function wrapSlots(slots) {
    var result = {};
    for (var key in slots) {
        if (slots[key]) {
            var slot = slots[key];
            var isObject = slot.slotType;
            var slotType = isObject ? slot.slotType : slot;
            var filter = isObject ? slot.filter : undefined;
            if (slot) {
                if (typeof slotType !== 'string' && slotType.__composable) {
                    result[key] = slotType.__composable;
                }
                else {
                    result[key] = wrapStockComponent(slotType, filter);
                }
            }
        }
    }
    return result;
}
//# sourceMappingURL=Composable.js.map